mod serde_blocktree;

use std::collections::HashMap;

/// A Block tagged with its version number.
#[allow(dead_code)]
enum VersionedBlock {
    V0(Block)
}

/// A container which binds together ciphertext along with the metadata needed to identify,
/// verify and decrypt it.
#[allow(dead_code)]
struct Block {
    /// A block is identified by this field, which defines its location in the tree.
    path: Path,
    /// This field contains a collection of `ReadCap`s indexed by the principal who holds them.
    /// `ReadCap`s are envelopments of the key used to encrypt this block.
    read_caps: HashMap<Principal, ReadCap>,
    /// This field is used to verify that the signer of this block had permission to write it.
    /// It contains a certificate chain that must lead back to the root key for the tree this block
    /// is part of.
    write_cap: WriteCap,
    /// The encrypted data contained in this block.
    body: SymCiphertext<Vec<u8>>,
    /// The contents of the block are covered by a digital signature contained in this field.  
    signature: Signature
}

/// The body of every non-leaf node in a tree contains this data structure.
#[allow(dead_code)]
struct Directory {
    /// The nodes that are attached to this block tree at this block.
    nodes: Vec<Principal>,
    /// The blocks that are descended from this one.
    children: HashMap<String, FragmentRecord>,
}

/// An envelopment of a key, which is tagged with the principal who the key is meant for.
#[allow(dead_code)]
struct ReadCap {
    /// The principal this `ReadCap` was issued to.
    issued_to: Principal,
    /// An encipherment of a block key using the public key of the principal.
    key: AsymCiphertext<Key>,
}

/// Verifies that a principal is authorized to write blocks in a tree.
#[allow(dead_code)]
struct WriteCap {
    /// The principal this `WriteCap` was issued to.
    issued_to: Principal,
    path: Path,
    chain: Vec<Certificate>,
    signature: Signature,
}

#[allow(dead_code)]
struct Certificate {
    issued_to: Principal,
    issued_by: Principal,
    signature: Signature,
    // TODO: Add expiration.
}

#[allow(dead_code)]
struct FragmentRecord {
    stored_by: Hash,
    serial: u32,
}

#[allow(dead_code)]
struct Fragment {
    path: String,
    serial: u32,
    body: Vec<u8>,
}

/// An identifier for a security principal, which is any entity that can be authenticated.
struct Principal(Hash);

/// Ciphertext which was produced using a symmetric algorithm.
struct SymCiphertext<T>(T);

/// Ciphertext that was produced using an asymmetric algorithm.
struct AsymCiphertext<T>(T);

/// An identifier for a block in a tree.
struct Path(Vec<String>);

#[allow(dead_code)]
enum Hash {
    Sha2_256([u8; 32]),
    Sha2_512([u8; 64]),
}

#[allow(dead_code)]
enum Signature {
    Ed25519([u8; 64]),
}

#[allow(dead_code)]
enum Key {
    Xsalsa20Poly1305([u8; 32]),
}

fn main() {
    println!("Hello, world!");
}
